@@ -19,6 +19,7 @@ package e2enode
 import (
 	"bufio"
 	"bytes"
+	"context"
 	"fmt"
 	"sort"
 	"strconv"
@@ -29,6 +30,7 @@ import (
 	"k8s.io/apimachinery/pkg/api/resource"
 
 	"k8s.io/kubernetes/test/e2e/framework"
+	e2epod "k8s.io/kubernetes/test/e2e/framework/pod"
 )
 
 type execCommand struct {
@@ -38,14 +40,23 @@ type execCommand struct {
 	StartDelay int
 	// Delay is how long the container should delay before exiting
 	Delay int
+	// LoopForever if set will cause the command to log once per second in a loop until
+	// terminated
+	LoopForever bool
 	// TerminationSeconds is the time it takes for the container before
 	// terminating if it catches SIGTERM.
 	TerminationSeconds int
+	// ContainerName is the name of the container to append the log. If empty,
+	// the name specified in ExecCommand will be used.
+	ContainerName string
 }
 
-// ExecCommand returns the command to execute in the container that implements execCommand and logs activities to a container
-// specific log that persists across container restarts.  The final log is written to /dev/termination-log so it can
-// be retrieved by the test harness after the container execution.
+// ExecCommand returns the command to execute in the container that implements
+// execCommand and logs activities to a container specific log that persists
+// across container restarts. The final log is written to container log so it
+// can be retrieved by the test harness during the container execution.
+// Log to /proc/1/fd/1 so that the lifecycle hook handler logs are captured as
+// well.
 func ExecCommand(name string, c execCommand) []string {
 	var cmd bytes.Buffer
 	// all outputs are in the format of:
@@ -54,25 +65,34 @@ func ExecCommand(name string, c execCommand) []string {
 	// The busybox time command doesn't support sub-second display. uptime displays in hundredths of a second, so we
 	// include both and use time since boot for relative ordering of file entries
 	timeCmd := "`date +%s` `cat /proc/uptime | awk '{print $1}'`"
-	containerLog := fmt.Sprintf("/persistent/%s.log", name)
+	containerName := name
+	if c.ContainerName != "" {
+		containerName = c.ContainerName
+	}
+	containerLog := fmt.Sprintf("/persistent/%s.log", containerName)
 
 	fmt.Fprintf(&cmd, "touch %s; ", containerLog)
-	fmt.Fprintf(&cmd, "cat %s >> /dev/termination-log; ", containerLog)
+	if c.ContainerName == "" {
+		fmt.Fprintf(&cmd, "cat %s >> /proc/1/fd/1; ", containerLog)
+	}
 
-	fmt.Fprintf(&cmd, "echo %s '%s Starting %d' | tee -a %s >> /dev/termination-log; ", timeCmd, name, c.StartDelay, containerLog)
-	fmt.Fprintf(&cmd, "_term() { sleep %d; echo %s '%s Exiting' | tee -a %s >> /dev/termination-log; exit %d; }; ", c.TerminationSeconds, timeCmd, name, containerLog, c.ExitCode)
+	fmt.Fprintf(&cmd, "echo %s '%s Starting %d' | tee -a %s >> /proc/1/fd/1; ", timeCmd, name, c.StartDelay, containerLog)
+	fmt.Fprintf(&cmd, "_term() { sleep %d; echo %s '%s Exiting' | tee -a %s >> /proc/1/fd/1; exit %d; }; ", c.TerminationSeconds, timeCmd, name, containerLog, c.ExitCode)
 	fmt.Fprintf(&cmd, "trap _term TERM; ")
 	if c.StartDelay != 0 {
 		fmt.Fprint(&cmd, sleepCommand(c.StartDelay))
 	}
 	// You can check started file to see if the container has started
 	fmt.Fprintf(&cmd, "touch started; ")
-	fmt.Fprintf(&cmd, "echo %s '%s Started' | tee -a %s >> /dev/termination-log; ", timeCmd, name, containerLog)
-	fmt.Fprintf(&cmd, "echo %s '%s Delaying %d' | tee -a %s >> /dev/termination-log; ", timeCmd, name, c.Delay, containerLog)
+	fmt.Fprintf(&cmd, "echo %s '%s Started' | tee -a %s >> /proc/1/fd/1; ", timeCmd, name, containerLog)
+	fmt.Fprintf(&cmd, "echo %s '%s Delaying %d' | tee -a %s >> /proc/1/fd/1; ", timeCmd, name, c.Delay, containerLog)
 	if c.Delay != 0 {
 		fmt.Fprint(&cmd, sleepCommand(c.Delay))
 	}
-	fmt.Fprintf(&cmd, "echo %s '%s Exiting'  | tee -a %s >> /dev/termination-log; ", timeCmd, name, containerLog)
+	if c.LoopForever {
+		fmt.Fprintf(&cmd, "while true; do echo %s '%s Looping' | tee -a %s >> /proc/1/fd/1 ; sleep 1 ; done; ", timeCmd, name, containerLog)
+	}
+	fmt.Fprintf(&cmd, "echo %s '%s Exiting'  | tee -a %s >> /proc/1/fd/1; ", timeCmd, name, containerLog)
 	fmt.Fprintf(&cmd, "exit %d", c.ExitCode)
 	return []string{"sh", "-c", cmd.String()}
 }
@@ -96,8 +116,8 @@ type containerOutputList []containerOutput
 
 func (o containerOutputList) String() string {
 	var b bytes.Buffer
-	for _, v := range o {
-		fmt.Fprintf(&b, "%s %f %s %s\n", v.timestamp, v.timeSinceBoot, v.containerName, v.command)
+	for i, v := range o {
+		fmt.Fprintf(&b, "%d) %s %f %s %s\n", i, v.timestamp, v.timeSinceBoot, v.containerName, v.command)
 	}
 	return b.String()
 }
@@ -111,18 +131,18 @@ func (o containerOutputList) RunTogether(lhs, rhs string) error {
 	rhsFinish := o.findIndex(rhs, "Finishing", 0)
 
 	if lhsStart == -1 {
-		return fmt.Errorf("couldn't find that %s ever started, got %v", lhs, o)
+		return fmt.Errorf("couldn't find that %s ever started, got\n%v", lhs, o)
 	}
 	if rhsStart == -1 {
-		return fmt.Errorf("couldn't find that %s ever started, got %v", rhs, o)
+		return fmt.Errorf("couldn't find that %s ever started, got\n%v", rhs, o)
 	}
 
 	if lhsFinish != -1 && rhsStart > lhsFinish {
-		return fmt.Errorf("expected %s to start before finishing %s, got %v", rhs, lhs, o)
+		return fmt.Errorf("expected %s to start before finishing %s, got\n%v", rhs, lhs, o)
 	}
 
 	if rhsFinish != -1 && lhsStart > rhsFinish {
-		return fmt.Errorf("expected %s to start before finishing %s, got %v", lhs, rhs, o)
+		return fmt.Errorf("expected %s to start before finishing %s, got\n%v", lhs, rhs, o)
 	}
 
 	return nil
@@ -133,14 +153,14 @@ func (o containerOutputList) StartsBefore(lhs, rhs string) error {
 	lhsStart := o.findIndex(lhs, "Started", 0)
 
 	if lhsStart == -1 {
-		return fmt.Errorf("couldn't find that %s ever started, got %v", lhs, o)
+		return fmt.Errorf("couldn't find that %s ever started, got\n%v", lhs, o)
 	}
 
 	// this works even for the same names (restart case)
 	rhsStart := o.findIndex(rhs, "Starting", lhsStart+1)
 
 	if rhsStart == -1 {
-		return fmt.Errorf("couldn't find that %s started after %s, got %v", rhs, lhs, o)
+		return fmt.Errorf("couldn't find that %s started after %s, got\n%v", rhs, lhs, o)
 	}
 	return nil
 }
@@ -150,14 +170,14 @@ func (o containerOutputList) DoesntStartAfter(lhs, rhs string) error {
 	rhsStart := o.findIndex(rhs, "Starting", 0)
 
 	if rhsStart == -1 {
-		return fmt.Errorf("couldn't find that %s ever started, got %v", rhs, o)
+		return fmt.Errorf("couldn't find that %s ever started, got\n%v", rhs, o)
 	}
 
 	// this works even for the same names (restart case)
 	lhsStart := o.findIndex(lhs, "Started", rhsStart+1)
 
 	if lhsStart != -1 {
-		return fmt.Errorf("expected %s to not start after %s, got %v", lhs, rhs, o)
+		return fmt.Errorf("expected %s to not start after %s, got\n%v", lhs, rhs, o)
 	}
 
 	return nil
@@ -168,14 +188,14 @@ func (o containerOutputList) ExitsBefore(lhs, rhs string) error {
 	lhsExit := o.findIndex(lhs, "Exiting", 0)
 
 	if lhsExit == -1 {
-		return fmt.Errorf("couldn't find that %s ever exited, got %v", lhs, o)
+		return fmt.Errorf("couldn't find that %s ever exited, got\n%v", lhs, o)
 	}
 
 	// this works even for the same names (restart case)
-	rhsExit := o.findIndex(rhs, "Starting", lhsExit+1)
+	rhsExit := o.findIndex(rhs, "Exiting", lhsExit+1)
 
 	if rhsExit == -1 {
-		return fmt.Errorf("couldn't find that %s starting before %s exited, got %v", rhs, lhs, o)
+		return fmt.Errorf("couldn't find that %s starting before %s exited (starting at idx %d), got\n%v", rhs, lhs, lhsExit+1, o)
 	}
 	return nil
 }
@@ -183,7 +203,7 @@ func (o containerOutputList) ExitsBefore(lhs, rhs string) error {
 // Starts returns an error if the container was not found to have started
 func (o containerOutputList) Starts(name string) error {
 	if idx := o.findIndex(name, "Started", 0); idx == -1 {
-		return fmt.Errorf("couldn't find that %s ever started, got %v", name, o)
+		return fmt.Errorf("couldn't find that %s ever started, got\n%v", name, o)
 	}
 	return nil
 }
@@ -191,7 +211,7 @@ func (o containerOutputList) Starts(name string) error {
 // DoesntStart returns an error if the container was found to have started
 func (o containerOutputList) DoesntStart(name string) error {
 	if idx := o.findIndex(name, "Started", 0); idx != -1 {
-		return fmt.Errorf("find %s started, but didn't expect to, got %v", name, o)
+		return fmt.Errorf("find %s started, but didn't expect to, got\n%v", name, o)
 	}
 	return nil
 }
@@ -199,7 +219,7 @@ func (o containerOutputList) DoesntStart(name string) error {
 // Exits returns an error if the container was not found to have exited
 func (o containerOutputList) Exits(name string) error {
 	if idx := o.findIndex(name, "Exiting", 0); idx == -1 {
-		return fmt.Errorf("couldn't find that %s ever exited, got %v", name, o)
+		return fmt.Errorf("couldn't find that %s ever exited, got\n%v", name, o)
 	}
 	return nil
 }
@@ -208,13 +228,13 @@ func (o containerOutputList) Exits(name string) error {
 func (o containerOutputList) HasRestarted(name string) error {
 	idx := o.findIndex(name, "Starting", 0)
 	if idx == -1 {
-		return fmt.Errorf("couldn't find that %s ever started, got %v", name, o)
+		return fmt.Errorf("couldn't find that %s ever started, got\n%v", name, o)
 	}
 
 	idx = o.findIndex(name, "Starting", idx+1)
 
 	if idx == -1 {
-		return fmt.Errorf("couldn't find that %s ever restarted, got %v", name, o)
+		return fmt.Errorf("couldn't find that %s ever restarted, got\n%v", name, o)
 	}
 
 	return nil
@@ -224,13 +244,13 @@ func (o containerOutputList) HasRestarted(name string) error {
 func (o containerOutputList) HasNotRestarted(name string) error {
 	idx := o.findIndex(name, "Starting", 0)
 	if idx == -1 {
-		return fmt.Errorf("couldn't find that %s ever started, got %v", name, o)
+		return fmt.Errorf("couldn't find that %s ever started, got\n%v", name, o)
 	}
 
 	idx = o.findIndex(name, "Starting", idx+1)
 
 	if idx != -1 {
-		return fmt.Errorf("found that %s restarted but wasn't expected to, got %v", name, o)
+		return fmt.Errorf("found that %s restarted but wasn't expected to, got\n%v", name, o)
 	}
 
 	return nil
@@ -264,24 +284,52 @@ func (o containerOutputList) findIndex(name string, command string, startIdx int
 	}
 	return -1
 }
+func (o containerOutputList) findLastIndex(name string, command string) int {
+	found := -1
+	for i, v := range o {
+		if v.containerName == name && v.command == command {
+			found = i
+		}
+	}
+	return found
+}
 
-// parseOutput combines the termination log from all of the init and regular containers and parses/sorts the outputs to
-// produce an execution log
-func parseOutput(pod *v1.Pod) containerOutputList {
+// TimeOfStart returns the time since the node boot in floating point seconds that the specified container started.
+func (o containerOutputList) TimeOfStart(name string) (float64, error) {
+	idx := o.findIndex(name, "Starting", 0)
+	if idx == -1 {
+		return 0.0, fmt.Errorf("couldn't find that %s ever started, got\n%v", name, o)
+	}
+	return o[idx].timeSinceBoot, nil
+}
+
+func (o containerOutputList) TimeOfLastLoop(name string) (float64, error) {
+	idx := o.findLastIndex(name, "Looping")
+	if idx == -1 {
+		return 0.0, fmt.Errorf("couldn't find that %s ever looped, got\n%v", name, o)
+	}
+	return o[idx].timeSinceBoot, nil
+}
+
+// parseOutput combines the container log from all of the init and regular
+// containers and parses/sorts the outputs to produce an execution log
+func parseOutput(ctx context.Context, f *framework.Framework, pod *v1.Pod) containerOutputList {
 	// accumulate all of our statuses
 	var statuses []v1.ContainerStatus
 	statuses = append(statuses, pod.Status.InitContainerStatuses...)
 	statuses = append(statuses, pod.Status.ContainerStatuses...)
+
 	var buf bytes.Buffer
 	for _, cs := range statuses {
-		// If the container is terminated but the reason is ContainerStatusUnknown,
-		// it means that the kubelet has overwritten the termination message. Read
-		// the LastTerminationState instead.
-		if cs.State.Terminated != nil && cs.State.Terminated.Reason != "ContainerStatusUnknown" {
-			buf.WriteString(cs.State.Terminated.Message)
-		} else if cs.LastTerminationState.Terminated != nil {
-			buf.WriteString(cs.LastTerminationState.Terminated.Message)
+		log, err := e2epod.GetPodLogs(ctx, f.ClientSet, f.Namespace.Name, pod.Name, cs.Name)
+		if err != nil {
+			framework.Logf("error getting logs for %s: %v", cs.Name, err)
+			log, err = e2epod.GetPreviousPodLogs(ctx, f.ClientSet, f.Namespace.Name, pod.Name, cs.Name)
+			if err != nil {
+				framework.Logf("error getting previous logs for %s: %v", cs.Name, err)
+			}
 		}
+		buf.WriteString(log)
 	}
 
 	// parse
