@@ -56,13 +56,12 @@ import (
 
 func TestSchedulingGates(t *testing.T) {
 	tests := []struct {
-		name           string
-		pods           []*v1.Pod
-		featureEnabled bool
-		want           []string
-		delete         []string
-		unschedulable  []string
-		rmGates        []string
+		name                  string
+		pods                  []*v1.Pod
+		featureEnabled        bool
+		want                  []string
+		rmPodsSchedulingGates []int
+		wantPostGatesRemoval  []string
 	}{
 		{
 			name: "feature disabled, regular pods",
@@ -99,7 +98,6 @@ func TestSchedulingGates(t *testing.T) {
 			},
 			featureEnabled: true,
 			want:           []string{"p2"},
-			unschedulable:  []string{"p1"},
 		},
 		{
 			name: "feature enabled, two pod carrying scheduling gates, and remove gates of one pod",
@@ -108,22 +106,10 @@ func TestSchedulingGates(t *testing.T) {
 				st.MakePod().Name("p2").SchedulingGates([]string{"bar"}).Container("pause").Obj(),
 				st.MakePod().Name("p3").Container("pause").Obj(),
 			},
-			featureEnabled: true,
-			want:           []string{"p3"},
-			unschedulable:  []string{"p1", "p2"},
-			rmGates:        []string{"p2"},
-		},
-		{
-			name: "gated pod schedulable after deleting the scheduled pod and removing gate",
-			pods: []*v1.Pod{
-				st.MakePod().Name("p1").SchedulingGates([]string{"foo"}).Container("pause").Obj(),
-				st.MakePod().Name("p2").Container("pause").Obj(),
-			},
-			featureEnabled: true,
-			want:           []string{"p2"},
-			delete:         []string{"p2"},
-			unschedulable:  []string{"p1"},
-			rmGates:        []string{"p1"},
+			featureEnabled:        true,
+			want:                  []string{"p3"},
+			rmPodsSchedulingGates: []int{1}, // remove gates of 'p2'
+			wantPostGatesRemoval:  []string{"p2"},
 		},
 	}
 
@@ -144,15 +130,6 @@ func TestSchedulingGates(t *testing.T) {
 			testutils.SyncSchedulerInformerFactory(testCtx)
 
 			cs, ns, ctx := testCtx.ClientSet, testCtx.NS.Name, testCtx.Ctx
-
-			// Create node, so we can schedule pods.
-			node := st.MakeNode().Name("node").Obj()
-			if _, err := cs.CoreV1().Nodes().Create(ctx, node, metav1.CreateOptions{}); err != nil {
-				t.Fatal("Failed to create node")
-
-			}
-
-			// Create pods.
 			for _, p := range tt.pods {
 				p.Namespace = ns
 				if _, err := cs.CoreV1().Pods(ns).Create(ctx, p, metav1.CreateOptions{}); err != nil {
@@ -168,44 +145,30 @@ func TestSchedulingGates(t *testing.T) {
 				t.Fatal(err)
 			}
 
-			go testCtx.Scheduler.Run(ctx)
-
-			// Verify pods scheduled.
-			for _, podName := range tt.want {
-				if err := wait.PollUntilContextTimeout(ctx, time.Millisecond*200, wait.ForeverTestTimeout, false, testutils.PodScheduled(cs, ns, podName).WithContext()); err != nil {
-					t.Fatalf("Failed to schedule %s", podName)
+			// Pop the expected pods out. They should be de-queueable.
+			for _, wantPod := range tt.want {
+				podInfo := testutils.NextPodOrDie(t, testCtx)
+				if got := podInfo.Pod.Name; got != wantPod {
+					t.Errorf("Want %v to be popped out, but got %v", wantPod, got)
 				}
 			}
 
-			// Delete pods, which triggers AssignedPodDelete event in the scheduling queue.
-			for _, podName := range tt.delete {
-				if err := cs.CoreV1().Pods(ns).Delete(ctx, podName, metav1.DeleteOptions{}); err != nil {
-					t.Fatalf("Error calling Delete on %s", podName)
-				}
-				if err := wait.PollUntilContextTimeout(ctx, time.Millisecond*200, wait.ForeverTestTimeout, false, testutils.PodDeleted(ctx, cs, ns, podName)); err != nil {
-					t.Fatalf("Failed to delete %s", podName)
-				}
+			if len(tt.rmPodsSchedulingGates) == 0 {
+				return
 			}
-
-			// Verify gated pods unschedulable.
-			for _, podName := range tt.unschedulable {
-				if err := wait.PollUntilContextTimeout(ctx, time.Millisecond*200, wait.ForeverTestTimeout, false, testutils.PodSchedulingGated(cs, ns, podName).WithContext()); err != nil {
-					t.Fatalf("Failed to verify unschedulable %s", podName)
-				}
-			}
-
 			// Remove scheduling gates from the pod spec.
-			for _, podName := range tt.rmGates {
+			for _, idx := range tt.rmPodsSchedulingGates {
 				patch := `{"spec": {"schedulingGates": null}}`
+				podName := tt.pods[idx].Name
 				if _, err := cs.CoreV1().Pods(ns).Patch(ctx, podName, types.StrategicMergePatchType, []byte(patch), metav1.PatchOptions{}); err != nil {
 					t.Fatalf("Failed to patch pod %v: %v", podName, err)
 				}
 			}
-
-			// Verify pods without gates scheduled.
-			for _, podName := range tt.rmGates {
-				if err := wait.PollUntilContextTimeout(ctx, time.Millisecond*200, wait.ForeverTestTimeout, false, testutils.PodScheduled(cs, ns, podName).WithContext()); err != nil {
-					t.Fatalf("Failed to schedule %s", podName)
+			// Pop the expected pods out. They should be de-queueable.
+			for _, wantPod := range tt.wantPostGatesRemoval {
+				podInfo := testutils.NextPodOrDie(t, testCtx)
+				if got := podInfo.Pod.Name; got != wantPod {
+					t.Errorf("Want %v to be popped out, but got %v", wantPod, got)
 				}
 			}
 		})
@@ -252,7 +215,7 @@ func TestCoreResourceEnqueue(t *testing.T) {
 	}
 
 	// Wait for the three pods to be present in the scheduling queue.
-	if err := wait.Poll(time.Millisecond*200, wait.ForeverTestTimeout, func() (bool, error) {
+	if err := wait.PollUntilContextTimeout(ctx, time.Millisecond*200, wait.ForeverTestTimeout, false, func(ctx context.Context) (bool, error) {
 		pendingPods, _ := testCtx.Scheduler.SchedulingQueue.PendingPods()
 		return len(pendingPods) == 3, nil
 	}); err != nil {
@@ -377,7 +340,7 @@ func TestCustomResourceEnqueue(t *testing.T) {
 	}
 
 	registry := frameworkruntime.Registry{
-		"fakeCRPlugin": func(_ runtime.Object, fh framework.Handle) (framework.Plugin, error) {
+		"fakeCRPlugin": func(_ context.Context, _ runtime.Object, fh framework.Handle) (framework.Plugin, error) {
 			return &fakeCRPlugin{}, nil
 		},
 	}
@@ -433,7 +396,7 @@ func TestCustomResourceEnqueue(t *testing.T) {
 	}
 
 	// Wait for the testing Pod to be present in the scheduling queue.
-	if err := wait.Poll(time.Millisecond*200, wait.ForeverTestTimeout, func() (bool, error) {
+	if err := wait.PollUntilContextTimeout(ctx, time.Millisecond*200, wait.ForeverTestTimeout, false, func(ctx context.Context) (bool, error) {
 		pendingPods, _ := testCtx.Scheduler.SchedulingQueue.PendingPods()
 		return len(pendingPods) == 1, nil
 	}); err != nil {
@@ -484,8 +447,8 @@ func TestCustomResourceEnqueue(t *testing.T) {
 func TestRequeueByBindFailure(t *testing.T) {
 	fakeBind := &firstFailBindPlugin{}
 	registry := frameworkruntime.Registry{
-		"firstFailBindPlugin": func(o runtime.Object, fh framework.Handle) (framework.Plugin, error) {
-			binder, err := defaultbinder.New(nil, fh)
+		"firstFailBindPlugin": func(ctx context.Context, o runtime.Object, fh framework.Handle) (framework.Plugin, error) {
+			binder, err := defaultbinder.New(ctx, nil, fh)
 			if err != nil {
 				return nil, err
 			}
@@ -539,7 +502,7 @@ func TestRequeueByBindFailure(t *testing.T) {
 	// 2. The pod should be enqueued to activeQ/backoffQ without any event.
 	// 3. The pod should be scheduled in the second binding try.
 	// Here, waiting until (3).
-	err := wait.Poll(200*time.Millisecond, wait.ForeverTestTimeout, testutils.PodScheduled(cs, ns, pod.Name))
+	err := wait.PollUntilContextTimeout(ctx, 200*time.Millisecond, wait.ForeverTestTimeout, false, testutils.PodScheduled(cs, ns, pod.Name))
 	if err != nil {
 		t.Fatalf("Expect pod-1 to be scheduled by the bind plugin: %v", err)
 	}
@@ -569,3 +532,134 @@ func (p *firstFailBindPlugin) Bind(ctx context.Context, state *framework.CycleSt
 
 	return p.defaultBinderPlugin.Bind(ctx, state, pod, nodename)
 }
+
+// TestRequeueByPermitRejection verify Pods failed by permit plugins in the binding cycle are
+// put back to the queue, according to the correct scheduling cycle number.
+func TestRequeueByPermitRejection(t *testing.T) {
+	defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.SchedulerQueueingHints, true)()
+	queueingHintCalledCounter := 0
+	fakePermit := &fakePermitPlugin{}
+	registry := frameworkruntime.Registry{
+		fakePermitPluginName: func(ctx context.Context, o runtime.Object, fh framework.Handle) (framework.Plugin, error) {
+			fakePermit = &fakePermitPlugin{
+				frameworkHandler: fh,
+				schedulingHint: func(logger klog.Logger, pod *v1.Pod, oldObj, newObj interface{}) (framework.QueueingHint, error) {
+					queueingHintCalledCounter++
+					return framework.Queue, nil
+				},
+			}
+			return fakePermit, nil
+		},
+	}
+	cfg := configtesting.V1ToInternalWithDefaults(t, configv1.KubeSchedulerConfiguration{
+		Profiles: []configv1.KubeSchedulerProfile{{
+			SchedulerName: pointer.String(v1.DefaultSchedulerName),
+			Plugins: &configv1.Plugins{
+				MultiPoint: configv1.PluginSet{
+					Enabled: []configv1.Plugin{
+						{Name: fakePermitPluginName},
+					},
+				},
+			},
+		}}})
+
+	// Use zero backoff seconds to bypass backoffQ.
+	testCtx := testutils.InitTestSchedulerWithOptions(
+		t,
+		testutils.InitTestAPIServer(t, "core-res-enqueue", nil),
+		0,
+		scheduler.WithPodInitialBackoffSeconds(0),
+		scheduler.WithPodMaxBackoffSeconds(0),
+		scheduler.WithProfiles(cfg.Profiles...),
+		scheduler.WithFrameworkOutOfTreeRegistry(registry),
+	)
+	testutils.SyncSchedulerInformerFactory(testCtx)
+
+	go testCtx.Scheduler.Run(testCtx.Ctx)
+
+	cs, ns, ctx := testCtx.ClientSet, testCtx.NS.Name, testCtx.Ctx
+	node := st.MakeNode().Name("fake-node").Obj()
+	if _, err := cs.CoreV1().Nodes().Create(ctx, node, metav1.CreateOptions{}); err != nil {
+		t.Fatalf("Failed to create Node %q: %v", node.Name, err)
+	}
+	// create a pod.
+	pod := st.MakePod().Namespace(ns).Name("pod-1").Container(imageutils.GetPauseImageName()).Obj()
+	if _, err := cs.CoreV1().Pods(ns).Create(ctx, pod, metav1.CreateOptions{}); err != nil {
+		t.Fatalf("Failed to create Pod %q: %v", pod.Name, err)
+	}
+
+	// update node label. (causes the NodeUpdate event)
+	node.Labels = map[string]string{"updated": ""}
+	if _, err := cs.CoreV1().Nodes().Update(ctx, node, metav1.UpdateOptions{}); err != nil {
+		t.Fatalf("Failed to add labels to the node: %v", err)
+	}
+
+	// create a pod to increment the scheduling cycle number in the scheduling queue.
+	// We can make sure NodeUpdate event, that has happened in the previous scheduling cycle, makes Pod to be enqueued to activeQ via the scheduling queue.
+	pod = st.MakePod().Namespace(ns).Name("pod-2").Container(imageutils.GetPauseImageName()).Obj()
+	if _, err := cs.CoreV1().Pods(ns).Create(ctx, pod, metav1.CreateOptions{}); err != nil {
+		t.Fatalf("Failed to create Pod %q: %v", pod.Name, err)
+	}
+
+	// reject pod-1 to simulate the failure in Permit plugins.
+	// This pod-1 should be enqueued to activeQ because the NodeUpdate event has happened.
+	fakePermit.frameworkHandler.IterateOverWaitingPods(func(wp framework.WaitingPod) {
+		if wp.GetPod().Name == "pod-1" {
+			wp.Reject(fakePermitPluginName, "fakePermitPlugin rejects the Pod")
+			return
+		}
+	})
+
+	// Wait for pod-2 to be scheduled.
+	err := wait.PollUntilContextTimeout(ctx, 200*time.Millisecond, wait.ForeverTestTimeout, false, func(ctx context.Context) (done bool, err error) {
+		fakePermit.frameworkHandler.IterateOverWaitingPods(func(wp framework.WaitingPod) {
+			if wp.GetPod().Name == "pod-2" {
+				wp.Allow(fakePermitPluginName)
+			}
+		})
+
+		return testutils.PodScheduled(cs, ns, "pod-2")(ctx)
+	})
+	if err != nil {
+		t.Fatalf("Expect pod-2 to be scheduled")
+	}
+
+	err = wait.PollUntilContextTimeout(ctx, 200*time.Millisecond, wait.ForeverTestTimeout, false, func(ctx context.Context) (done bool, err error) {
+		pod1Found := false
+		fakePermit.frameworkHandler.IterateOverWaitingPods(func(wp framework.WaitingPod) {
+			if wp.GetPod().Name == "pod-1" {
+				pod1Found = true
+				wp.Allow(fakePermitPluginName)
+			}
+		})
+		return pod1Found, nil
+	})
+	if err != nil {
+		t.Fatal("Expect pod-1 to be scheduled again")
+	}
+
+	if queueingHintCalledCounter != 1 {
+		t.Fatalf("Expected the scheduling hint to be called 1 time, but %v", queueingHintCalledCounter)
+	}
+}
+
+type fakePermitPlugin struct {
+	frameworkHandler framework.Handle
+	schedulingHint   framework.QueueingHintFn
+}
+
+const fakePermitPluginName = "fakePermitPlugin"
+
+func (p *fakePermitPlugin) Name() string {
+	return fakePermitPluginName
+}
+
+func (p *fakePermitPlugin) Permit(ctx context.Context, state *framework.CycleState, _ *v1.Pod, _ string) (*framework.Status, time.Duration) {
+	return framework.NewStatus(framework.Wait), wait.ForeverTestTimeout
+}
+
+func (p *fakePermitPlugin) EventsToRegister() []framework.ClusterEventWithHint {
+	return []framework.ClusterEventWithHint{
+		{Event: framework.ClusterEvent{Resource: framework.Node, ActionType: framework.UpdateNodeLabel}, QueueingHintFn: p.schedulingHint},
+	}
+}
